# Generated by Django 3.2.5 on 2021-07-24 17:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0017_auto_20210724_2306'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='assessment',
            options={'permissions': [('preview_assessment', 'Can preview assessment')]},
        ),
        migrations.AlterField(
            model_name='assessment',
            name='duration',
            field=models.FloatField(help_text='This duration will be in minutes'),
        ),
        migrations.AlterField(
            model_name='assessment',
            name='score',
            field=models.FloatField(help_text='This will auto'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-25 11:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0022_auto_20210725_1700'),
    ]

    operations = [
        migrations.AddField(
            model_name='assessment',
            name='pass_score',
            field=models.FloatField(default=10),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-20 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0011_alter_candidatejob_step'),
    ]

    operations = [
        migrations.AlterField(
            model_name='candidatejob',
            name='step',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



from django.contrib import admin
from django import forms
from django.contrib.admin.widgets import FilteredSelectMultiple

from job_board.models import JobSummery, Job, JobAdditionalField, Assessment


class JobSummeryInline(admin.StackedInline):
    model = JobSummery
    min_num = 1


class AdditionalFieldInline(admin.TabularInline):
    model = JobAdditionalField

    def get_extra(self, request, obj=None, **kwargs):
        if obj:
            return 0
        return 1


class JobForm(forms.ModelForm):
    assessments = forms.ModelMultipleChoiceField(
        queryset=Assessment.objects.all(),
        widget=FilteredSelectMultiple(verbose_name='assessments', is_stacked=False)
    )

    class Meta:
        model = Job
        fields = '__all__'


@admin.register(Job)
class JobAdmin(admin.ModelAdmin):
    inlines = (JobSummeryInline, AdditionalFieldInline)
    list_display = ('title', 'job_summery',)
    form = JobForm



# Generated by Django 3.2.5 on 2021-07-20 13:42

from django.db import migrations, models
import tinymce.models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0009_jobadditionalfield'),
    ]

    operations = [
        migrations.AddField(
            model_name='candidatejob',
            name='mcq_exam_started_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='candidatejob',
            name='step',
            field=models.IntegerField(default=0, max_length=11),
        ),
        migrations.AlterField(
            model_name='job',
            name='additional_requirement',
            field=tinymce.models.HTMLField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='job',
            name='compensation',
            field=tinymce.models.HTMLField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='job',
            name='educational_requirement',
            field=tinymce.models.HTMLField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='job',
            name='job_context',
            field=tinymce.models.HTMLField(max_length=500),
        ),
        migrations.AlterField(
            model_name='job',
            name='job_description',
            field=tinymce.models.HTMLField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='job',
            name='job_responsibility',
            field=tinymce.models.HTMLField(blank=True, null=True),
        ),
    ]



from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.mixins import CreateModelMixin
from rest_framework.response import Response
from rest_framework.views import APIView

from job_board.auth.CandidateAuth import CandidateAuth, CredentialsSerializer
from job_board.models import Candidate
from job_board.serializers.candidate_serializer import CandidateSerializer, CandidateUpdateSerializer
from job_board.serializers.password_reset import SendOTPSerializer, ResetPasswordSerializer


class Registration(CreateModelMixin, GenericAPIView):
    """
    Candidate registration requires a form data with
    """
    queryset = Candidate.objects.all()
    serializer_class = CandidateSerializer

    def post(self, request, *args, **kwargs):
        return self.create(request, *args, **kwargs)


class Login(GenericAPIView, CreateModelMixin):
    """
    Candidate Login

    candidate only can able to login with email & password

    send a post request with a valid json format { "email" : "<your@email>", "password": "<your password>" }
    """

    serializer_class = CredentialsSerializer

    def post(self, request, format=None):
        auth = CandidateAuth()
        return auth.auth_token(request)


class User(APIView):
    """
    Candidate information
    TODO : update profile update will be in post method
    """
    authentication_classes = [CandidateAuth]

    def get(self, request, format=None):
        serialize = CandidateSerializer(request.user, context={"request": request})
        return Response(serialize.data)

    def post(self, request, format=None):
        serialize = CandidateUpdateSerializer(data=request.data)
        if serialize.is_valid():
            serialize.update(instance=request.user, validated_data=serialize.validated_data)
            return Response(serialize.data)
        return Response(serialize.errors, status=status.HTTP_400_BAD_REQUEST)


class SendOTP(GenericAPIView, CreateModelMixin):
    serializer_class = SendOTPSerializer
    queryset = Candidate.objects.all()

    def post(self, request, *args, **kwargs):
        self.create(request, *args, **kwargs)
        return Response({'message': 'OTP has been sent'}, status=status.HTTP_200_OK)


class ResetPasswordView(GenericAPIView, CreateModelMixin):
    serializer_class = ResetPasswordSerializer

    def post(self, request, *args, **kwargs):
        self.create(request, *args, **kwargs)
        return Response({'message': 'Candidate password has been updated successfully'})


class ChangeCandidatePassword(GenericAPIView):
    authentication_classes = [CandidateAuth]



# Generated by Django 3.2.5 on 2021-07-28 14:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0028_remove_assessment_score'),
    ]

    operations = [
        migrations.RenameField(
            model_name='assessment',
            old_name='pass_score',
            new_name='pass_percentage',
        ),
    ]



from collections import OrderedDict

from django.db.models import Sum
from django.utils import timezone
from rest_framework import serializers
from rest_framework.serializers import ModelSerializer

from job_board.models import Assessment, AssessmentAnswer, AssessmentQuestion, CandidateJob, CandidateAssessmentAnswer, \
    CandidateAssessment
from job_board.serializers.candidate_serializer import CandidateJobSerializer


class AssessmentSerializer(ModelSerializer):
    class Meta:
        model = Assessment
        fields = ['title', 'slug', 'description', 'score', 'pass_score', 'duration', 'type']


class AssessmentAnswerSerializer(ModelSerializer):
    class Meta:
        model = AssessmentAnswer
        fields = ('id', 'title')


class AssessmentQuestionSerializer(ModelSerializer):
    answers = AssessmentAnswerSerializer(many=True, read_only=True)

    class Meta:
        model = AssessmentQuestion
        fields = ('id', 'title', 'type', 'answers')


class CandidateAssessmentSerializer(serializers.ModelSerializer):
    assessment = AssessmentSerializer(many=False)
    created_at = serializers.DateTimeField(format='%B %d, %Y')
    updated_at = serializers.DateTimeField(format='%B %d, %Y')
    candidate_job = CandidateJobSerializer(many=False)
    time_spend = serializers.CharField()

    class Meta:
        model = CandidateAssessment
        fields = ('unique_id', 'created_at', 'updated_at', 'exam_started_at', 'exam_end_at', 'score', 'time_spend',
                  'evaluation_url', 'step', 'candidate_job', 'assessment')


def valid_uuid(value):
    candidate_assessment = CandidateAssessment.objects.filter(unique_id=value).first()
    if not candidate_assessment:
        raise serializers.ValidationError('We could not found any assessment in your given uuid')
    # TODO : this section should uncommented in production
    # if candidate_assessment.time_spend == 'time_up':
    #     raise serializers.ValidationError(f'{candidate_assessment.assessment} has been expired')


class GivenAssessmentAnswerSerializer(serializers.Serializer):
    """
    Most complicated part, Handle with care
    TODO : i need to modify the comment and it should be elaborate
    """
    uuid = serializers.UUIDField(validators=[valid_uuid])
    question_id = serializers.IntegerField(min_value=1)
    answers = serializers.ListField(child=serializers.IntegerField(min_value=1), min_length=1)

    candidate_assessment = CandidateAssessment
    candidate_job = CandidateJob
    question = AssessmentQuestion
    candidate_answer = CandidateAssessmentAnswer

    def validate(self, data: OrderedDict):
        self.candidate_assessment = CandidateAssessment.objects.filter(unique_id__exact=data['uuid']).first()
        self.candidate_job = self.candidate_assessment.candidate_job
        self.question = AssessmentQuestion.objects.get(pk=data['question_id'])
        if self.question.type == 'single_choice' and len(data['answers']) > 1:
            raise serializers.ValidationError(
                {
                    'answers':
                        f'{self.question.get_type_display()} allow single answer, your answer {len(data["answers"])}'
                }
            )
        return data

    def create(self, validated_data):
        assessment_answer = AssessmentAnswer.objects.filter(pk__in=validated_data['answers'],
                                                            assessment_question_id__exact=validated_data[
                                                                'question_id']
                                                            ).all()
        candidate_answer = CandidateAssessmentAnswer.objects.filter(question=self.question,
                                                                    candidate_job=self.candidate_job).first()
        if not candidate_answer:
            self._create_answer(assessment_answer=assessment_answer)
            return validated_data
        raise serializers.ValidationError({'message': 'This answer has been taken already'})

    def _create_answer(self, assessment_answer):
        candidate_answer = CandidateAssessmentAnswer()
        candidate_answer.candidate_job = self.candidate_job
        candidate_answer.question = self.question
        candidate_answer.answers = AssessmentAnswerSerializer(assessment_answer, many=True).data
        candidate_answer.total_score = self.question.score
        candidate_answer.score_achieve = assessment_answer.aggregate(score_achieve=Sum('score'))['score_achieve']
        candidate_answer.save()
        self._step_increment()
        self._add_mcq_mark(candidate_answer.score_achieve)

    def _step_increment(self):
        self.candidate_assessment.step['current_step'] += 1
        self.candidate_assessment.save()

    def _add_mcq_mark(self, score):
        self.candidate_assessment.score += score
        self.candidate_assessment.save()



# Generated by Django 3.2.5 on 2021-07-02 06:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0004_auto_20210629_1947'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='assessment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='assessment', to='job_board.assessment'),
        ),
        migrations.AlterField(
            model_name='jobsummery',
            name='job',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='job_summery', to='job_board.job'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-27 16:34

from django.db import migrations, models
import tinymce.models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0025_assessment_open_to_start'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assessment',
            name='score',
            field=models.FloatField(default=0, help_text='This will auto'),
        ),
        migrations.AlterField(
            model_name='assessmentanswer',
            name='title',
            field=tinymce.models.HTMLField(),
        ),
        migrations.AlterField(
            model_name='assessmentquestion',
            name='title',
            field=tinymce.models.HTMLField(),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-24 16:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0015_candidateassessment'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='candidatejob',
            name='mcq_exam_score',
        ),
        migrations.RemoveField(
            model_name='candidatejob',
            name='mcq_exam_started_at',
        ),
        migrations.RemoveField(
            model_name='candidatejob',
            name='step',
        ),
        migrations.RemoveField(
            model_name='candidatejob',
            name='viva_exam_score',
        ),
        migrations.RemoveField(
            model_name='candidatejob',
            name='written_exam_score',
        ),
        migrations.AlterField(
            model_name='candidateassessment',
            name='assessment',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='job_board.assessment'),
        ),
    ]



import datetime
from datetime import timedelta

from django.http import Http404
from django.utils import timezone
from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.mixins import RetrieveModelMixin, ListModelMixin, CreateModelMixin
from rest_framework.response import Response
from rest_framework.views import APIView

from config import settings
from job_board.auth.CandidateAuth import CandidateAuth
from job_board.models import Job, CandidateJob, Candidate
from job_board.serializers.job_serializer import JobSerializer
from job_board.serializers.candidate_serializer import CandidateJobSerializer, CandidateJobApplySerializer


class GenericJobView(GenericAPIView):
    queryset = Job.objects.all()
    serializer_class = JobSerializer

    class Meta:
        abstract = True


class JobList(ListModelMixin, GenericJobView):

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class JobRetrieve(RetrieveModelMixin, GenericJobView):
    lookup_field = 'slug'

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class CandidateJobView(APIView):
    """
    List all candidate jobs, or create a new candidate job.
    """

    authentication_classes = [CandidateAuth]

    def post(self, request, *args, **kwargs):
        serializer = CandidateJobApplySerializer(data=request.data)
        if serializer.is_valid():
            serializer.validated_data['candidate'] = request.user
            serializer.validated_data['job'] = self.get_object(serializer.validated_data['job_slug'])
            if not self.__applied_before(serializer.validated_data['candidate'], serializer.validated_data['job']):
                serializer.save()
                return Response({'success': 'You job application has been submitted successfully'})
            return Response({'message': 'You applied for the same position less then 90 days before, '
                                        'unfortunately you cannot re-apply before 90 days of your last application'},
                            status=status.HTTP_403_FORBIDDEN)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def get(self, request, *args, **kwargs):
        candidate_jobs = CandidateJob.objects.filter(candidate_id=request.user.id).order_by('-id').all()
        candidate_job_serialize = CandidateJobSerializer(candidate_jobs, many=True)
        return Response(candidate_job_serialize.data)

    def get_object(self, slug):
        try:
            return Job.objects.get(slug__exact=slug)
        except Job.DoesNotExist:
            raise Http404

    def __applied_before(self, candidate: Candidate, job: Job):
        days_before = timezone.now() - timedelta(days=settings.APPLY_SAME_JOB_AFTER or 90)
        if CandidateJob.objects.filter(candidate=candidate, job=job, created_at__gte=days_before):
            return True
        return False



# Generated by Django 3.2.5 on 2021-07-20 19:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0012_alter_candidatejob_step'),
    ]

    operations = [
        migrations.CreateModel(
            name='CandidateAssessmentAnswer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('total_score', models.FloatField()),
                ('answers', models.JSONField()),
                ('score_achieve', models.FloatField()),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='job_board.assessmentquestion')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-19 10:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0007_auto_20210714_1637'),
    ]

    operations = [
        migrations.CreateModel(
            name='ResetPassword',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('email', models.EmailField(max_length=254)),
                ('otp', models.CharField(max_length=10)),
                ('otp_expire_at', models.DateTimeField()),
                ('otp_used_at', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-20 08:28

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_userforeignkey.models.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('job_board', '0008_resetpassword'),
    ]

    operations = [
        migrations.CreateModel(
            name='JobAdditionalField',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255)),
                ('required', models.BooleanField(default=True)),
                ('validation_regx', models.CharField(blank=True, max_length=255, null=True)),
                ('created_by', django_userforeignkey.models.fields.UserForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='job_board_jobadditionalfield_related', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
                ('job', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='additional_fields', to='job_board.job')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import jwt
from django.contrib.auth import hashers
from rest_framework import serializers
from rest_framework.authentication import BaseAuthentication, get_authorization_header
from rest_framework.exceptions import AuthenticationFailed
from rest_framework.fields import CharField, EmailField
from rest_framework.response import Response
from rest_framework.serializers import Serializer
from rest_framework.status import HTTP_401_UNAUTHORIZED, HTTP_400_BAD_REQUEST

from job_board.models import Candidate
from job_board.serializers.candidate_serializer import CandidateSerializer


class CredentialsSerializer(Serializer):
    """
    Credentials Serializer
    use for request validation and json response

    @type Serializer : Serializer
    """
    email = CharField()
    password = CharField(max_length=40, min_length=5)


class CandidateAuth(BaseAuthentication):
    """
    Candidate auth
    This class is responsible to create jwt auth token using given credentials
    also it is responsible to register user
    """
    STRONG_SECRET = "rifat"
    ENCRYPT_ALGORITHMS = "HS256"

    # candidate model
    candidate = None

    def authenticate(self, request):
        """
        Validate the given jwt token and process next if matched
        @type request: object
        """
        token = get_authorization_header(request).split()
        if token:
            user = jwt.decode(token[1], self.STRONG_SECRET, algorithms=self.ENCRYPT_ALGORITHMS)
            if user:
                user = Candidate.objects.get(pk=user['id'])
                return user, None
            else:
                raise serializers.ValidationError({'token': f'your token {token[1]} seems not valid'})
        else:
            raise serializers.ValidationError({'token': 'Token Not found'})

    def auth_token(self, request):
        """
        Create auth token using given credentials
        if the data is valid and password is matched with given email address

        @param request:
        @return:
        """
        serializer = CredentialsSerializer(data=request.data)
        if serializer.is_valid() and self.__match_credentials(serializer):
            return self.__generate_token(self.candidate)
        return Response(serializer.errors, status=HTTP_400_BAD_REQUEST)

    def __match_credentials(self, serializer: CredentialsSerializer):
        """
        Match given credentials from serializers
        @raise error if given username / email is not found
        @raise error if given password is not matched with given email / username

        @param serializer:
        @return:
        """
        try:
            candidate = Candidate.objects.get(email__exact=serializer.data['email'])
            if hashers.check_password(serializer.data['password'], candidate.password):
                self.candidate = candidate
                return True
            raise AuthenticationFailed('Credentials does not matched')
        except:
            raise AuthenticationFailed('Credentials does not matched')

    def __generate_token(self, candidate: Candidate):
        """
        Generate JWT token

        @param candidate:
        @return json:
        """
        user_serializer = CandidateSerializer(candidate)
        encoded = jwt.encode(user_serializer.data, self.STRONG_SECRET, algorithm=self.ENCRYPT_ALGORITHMS)
        return Response({'_token': encoded})



# Generated by Django 3.2.5 on 2021-07-28 09:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0026_auto_20210727_2234'),
    ]

    operations = [
        migrations.AddField(
            model_name='jobsummery',
            name='salary_range',
            field=models.CharField(default=4, max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='jobsummery',
            name='experience',
            field=models.CharField(max_length=255),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-28 14:42

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_userforeignkey.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Assessment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255)),
                ('slug', models.SlugField()),
                ('score', models.FloatField()),
                ('duration', models.FloatField()),
                ('description', models.TextField()),
                ('created_by', django_userforeignkey.models.fields.UserForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='job_board_assessment_related', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Candidate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('full_name', models.CharField(max_length=150)),
                ('email', models.EmailField(max_length=40, unique=True)),
                ('phone', models.CharField(max_length=11, unique=True)),
                ('password', models.CharField(max_length=255)),
                ('avatar', models.ImageField(blank=True, null=True, upload_to='candidate/avatar/')),
                ('cv', models.FileField(upload_to='hr/%Y/%m/')),
                ('status', models.CharField(choices=[('active', 'Active'), ('banned', 'Banned')], default='active', max_length=10)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=155)),
                ('slug', models.SlugField(max_length=255)),
                ('job_context', models.TextField(max_length=500)),
                ('job_description', models.TextField(blank=True, null=True)),
                ('job_responsibility', models.TextField(blank=True, null=True)),
                ('educational_requirement', models.TextField(blank=True, null=True)),
                ('additional_requirement', models.TextField(blank=True, null=True)),
                ('compensation', models.TextField(blank=True, null=True)),
                ('created_by', django_userforeignkey.models.fields.UserForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='job_board_job_related', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='JobSummery',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('application_deadline', models.DateField()),
                ('experience', models.IntegerField(help_text='Experience in year')),
                ('job_type', models.CharField(choices=[('full_time', 'Full Time'), ('part_time', 'Part Time'), ('contractual', 'Contractual')], max_length=22)),
                ('vacancy', models.IntegerField()),
                ('created_by', django_userforeignkey.models.fields.UserForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='job_board_jobsummery_related', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
                ('job', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='job_board.job')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CandidateJob',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('mcq_exam_score', models.FloatField(default=0)),
                ('written_exam_score', models.FloatField(default=0)),
                ('viva_exam_score', models.FloatField(default=0)),
                ('candidate', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='job_board.candidate')),
                ('job', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, to='job_board.job')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AssessmentQuestion',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255)),
                ('score', models.FloatField()),
                ('type', models.CharField(choices=[('single_choice', 'Single Choice'), ('multiple_choice', 'Multiple Choice')], max_length=20)),
                ('assessment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='job_board.assessment')),
                ('created_by', django_userforeignkey.models.fields.UserForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='job_board_assessmentquestion_related', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AssessmentAnswer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.TextField()),
                ('score', models.FloatField()),
                ('correct', models.BooleanField(default=False)),
                ('assessment_question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='job_board.assessmentquestion')),
                ('created_by', django_userforeignkey.models.fields.UserForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='job_board_assessmentanswer_related', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-28 16:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0029_rename_pass_score_assessment_pass_percentage'),
    ]

    operations = [
        migrations.AddField(
            model_name='candidateassessment',
            name='can_open_after',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from django.utils import timezone
from rest_framework import serializers

from job_board.models import ResetPassword, Candidate


class ValidCandidateEmail:
    def __call__(self, value):
        if not Candidate.objects.filter(email=value).first():
            raise serializers.ValidationError(
                'Your given email is not found in candidate list, please insert a valid email address')


class SendOTPSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(validators=[ValidCandidateEmail()])

    class Meta:
        model = ResetPassword
        fields = ['email']


class ResetPasswordSerializer(serializers.Serializer):
    email = serializers.EmailField(validators=[ValidCandidateEmail()])
    otp = serializers.CharField(min_length=6, max_length=6)
    password = serializers.CharField(min_length=6, max_length=40)

    def validate(self, data):
        if ResetPassword.objects.filter(
                otp_used_at__isnull=True,
                email__exact=data['email'],
                otp__exact=data['otp'],
                otp_expire_at__gte=timezone.now()).last():
            return data
        raise serializers.ValidationError({'otp': 'OTP is not correct or expire'})

    def update(self, instance, validated_data):
        pass

    def create(self, validated_data):
        candidate = Candidate.objects.filter(email__exact=validated_data['email']).first()
        candidate.password = validated_data['password']
        candidate.save()

        pass_reset = ResetPassword.objects.filter(otp__exact=validated_data['otp']).last()
        pass_reset.otp_used_at = timezone.now()
        pass_reset.save()
        return validated_data



# Generated by Django 3.2.5 on 2021-07-25 10:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0019_auto_20210725_1648'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='candidateassessment',
            name='exam_type',
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-29 13:47

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0003_alter_job_assessment'),
    ]

    operations = [
        migrations.AddField(
            model_name='candidatejob',
            name='unique_id',
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='assessmentanswer',
            name='assessment_question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='job_board.assessmentquestion'),
        ),
        migrations.AlterField(
            model_name='job',
            name='assessment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='job_board.assessment'),
        ),
    ]



import datetime
import random
from datetime import timedelta

from django.contrib.auth import hashers
from django.contrib.humanize.templatetags.humanize import naturalday
from django.core.exceptions import ValidationError
from django.db import models
# from gdstorage.storage import GoogleDriveStorage
from django.db.models import Sum
from django.utils import timezone
from tinymce.models import HTMLField
import uuid

# Create your models here.
from job_board.tasks import send_otp
from django_userforeignkey.models.fields import UserForeignKey

class AuthorMixin(models.Model):
    created_by = UserForeignKey(auto_user_add=True, verbose_name="Created By",
                                related_name="%(app_label)s_%(class)s_related")

    class Meta:
        abstract = True

class TimeStampMixin(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Assessment(AuthorMixin, TimeStampMixin):
    TYPE_CHOICE = (
        ('mcq', 'MCQ Examination'),
        ('written', 'Written Examination'),
        ('viva', 'Viva/Oral Examination'),
    )

    title = models.CharField(max_length=255)
    slug = models.SlugField()
    pass_percentage = models.FloatField()
    duration = models.FloatField(help_text='This duration will be in minutes')
    description = HTMLField()
    type = models.CharField(max_length=40, choices=TYPE_CHOICE, default='mcq')
    open_to_start = models.BooleanField(default=False)

    def __str__(self):
        return self.title

    @property
    def duration_display(self):
        to_human = f'{self.duration} Minutes'
        if self.duration > 60:
            in_hour = str(round(self.duration / 60, 2)).split(".")
            to_human = f'{in_hour[0]} Hour {in_hour[1]} Minutes'
        return to_human

    @property
    def score(self):
        score = self.assessmentquestion_set.all().aggregate(Sum('score'))['score__sum']
        return score if score else 0

    @property
    def pass_score(self):
        return (float(self.score) * float(self.pass_percentage)) / 100

    class Meta:
        permissions = [
            ("preview_assessment", "Can preview assessment")
        ]


class AssessmentQuestion(AuthorMixin, TimeStampMixin):
    TYPE = (
        ('single_choice', 'Single Choice'),
        ('multiple_choice', 'Multiple Choice'),
    )
    assessment = models.ForeignKey(Assessment, on_delete=models.CASCADE)
    title = HTMLField()
    score = models.FloatField()
    type = models.CharField(max_length=20, choices=TYPE)


class AssessmentAnswer(AuthorMixin, TimeStampMixin):
    title = HTMLField()
    score = models.FloatField()
    correct = models.BooleanField(default=False)
    assessment_question = models.ForeignKey(AssessmentQuestion, related_name='answers', on_delete=models.CASCADE)


class Job(AuthorMixin, TimeStampMixin):
    title = models.CharField(max_length=155)
    slug = models.SlugField(max_length=255)
    job_context = HTMLField(max_length=500)
    job_description = HTMLField(null=True, blank=True)
    job_responsibility = HTMLField(null=True, blank=True)
    educational_requirement = HTMLField(null=True, blank=True)
    additional_requirement = HTMLField(null=True, blank=True)
    compensation = HTMLField(null=True, blank=True)
    assessments = models.ManyToManyField(Assessment)

    def __str__(self):
        return self.title


class JobSummery(AuthorMixin, TimeStampMixin):
    JOB_TYPE = (
        ('full_time', 'Full Time'),
        ('part_time', 'Part Time'),
        ('contractual', 'Contractual')
    )
    job = models.OneToOneField(Job, on_delete=models.CASCADE, related_name='job_summery')
    application_deadline = models.DateField()
    experience = models.CharField(max_length=255)
    job_type = models.CharField(max_length=22, choices=JOB_TYPE)
    vacancy = models.IntegerField()
    salary_range = models.CharField(max_length=255)

    def __str__(self):
        return f'{naturalday(self.application_deadline)} | {self.vacancy} | {self.job_type}'


class JobAdditionalField(TimeStampMixin, AuthorMixin):
    job = models.ForeignKey(Job, on_delete=models.CASCADE, related_name='additional_fields')
    title = models.CharField(max_length=255)
    required = models.BooleanField(default=True)
    validation_regx = models.CharField(max_length=255, null=True, blank=True)


# go_storage = GoogleDriveStorage()

def candidate_email_path(instance, filename):
    return 'hr/candidate/{0}/{1}'.format(instance.email, filename)


class Candidate(TimeStampMixin):
    STATUS_CHOICE = (
        ('active', 'Active'),
        ('banned', 'Banned')
    )
    full_name = models.CharField(max_length=150)
    email = models.EmailField(max_length=40, unique=True)
    email_otp = models.CharField(max_length=10, null=True, blank=True)
    email_verified_at = models.DateField(null=True, blank=True)
    phone = models.CharField(max_length=11, unique=True)
    password = models.CharField(max_length=255)
    avatar = models.ImageField(upload_to='candidate/avatar/', null=True, blank=True)
    cv = models.FileField(upload_to=candidate_email_path)
    status = models.CharField(max_length=10, choices=STATUS_CHOICE, default='active')

    def save(self, *args, **kwargs):
        self.password = hashers.make_password(self.password, salt='mediusware_hr')
        super().save(*args, **kwargs)

    def __str__(self):
        return self.full_name


class CandidateJob(TimeStampMixin):
    unique_id = models.UUIDField(default=uuid.uuid4, editable=False)
    candidate = models.ForeignKey(Candidate, on_delete=models.CASCADE)
    job = models.ForeignKey(Job, on_delete=models.RESTRICT)
    expected_salary = models.FloatField()
    additional_message = models.TextField(null=True, blank=True)

    def save(self, *ages, **kwargs):
        super(CandidateJob, self).save(*ages, **kwargs)
        for assessment in self.job.assessments.all():
            candidate_assessment = CandidateAssessment()
            candidate_assessment.candidate_job = self
            candidate_assessment.assessment = assessment
            candidate_assessment.save()
        # TODO : Schedule mail for assessment

    def __str__(self):
        return f'{self.candidate.full_name} | {self.job.title}'


class CandidateAssessment(TimeStampMixin):
    unique_id = models.UUIDField(default=uuid.uuid4, editable=False)
    candidate_job = models.ForeignKey(CandidateJob, on_delete=models.CASCADE, related_name='candidate_job')
    assessment = models.ForeignKey(Assessment, on_delete=models.CASCADE)
    can_start_after = models.DateTimeField(null=True, blank=True)
    exam_started_at = models.DateTimeField(null=True, blank=True)
    exam_end_at = models.DateTimeField(null=True, blank=True)
    score = models.FloatField(default=0)
    evaluation_url = models.CharField(null=True, blank=True, max_length=255)
    step = models.JSONField(null=True, blank=True)
    note = models.TextField(null=True, blank=True)

    # TODO : send mail to candidate when can start after is present
    # It can be auto mated with cron job

    @property
    def time_spend(self):
        if self.exam_end_at is not None:
            now = timezone.now()
            if self.exam_end_at >= now:
                return now - self.exam_started_at
            return 'time_up'
        return 'not_started'

    @property
    def status(self):
        if self.step is None:
            return '-'
        if self.step['current_step'] == len(self.step['question_ids']):
            return 'complete'
        return 'processing'

    @property
    def result(self):
        if not self.step:
            return '-'
        if self.score >= self.assessment.pass_score:
            return 'pass'
        return 'fail'


class CandidateAssessmentAnswer(TimeStampMixin):
    candidate_job = models.ForeignKey(CandidateJob, on_delete=models.CASCADE)
    question = models.ForeignKey(AssessmentQuestion, on_delete=models.CASCADE)
    total_score = models.FloatField()
    answers = models.JSONField()
    score_achieve = models.FloatField()


class ResetPassword(TimeStampMixin):
    email = models.EmailField()
    otp = models.CharField(max_length=10)
    otp_expire_at = models.DateTimeField()
    otp_used_at = models.DateTimeField(null=True, blank=True)

    def clean_fields(self, exclude=None):
        super(ResetPassword, self).clean_fields(exclude=['otp', 'otp_expire_at'])
        if not Candidate.objects.filter(email__exact=self.email).first():
            raise ValidationError(
                {'email': 'Your given email is not found in candidate list, please insert a valid email address'})

    def save(self, *args, **kwargs):
        if self.otp_used_at is None:
            self.otp = random.randrange(100000, 999999, 6)
            self.otp_expire_at = timezone.now() + timedelta(minutes=15)
            send_otp(self.otp, self.email)
        super(ResetPassword, self).save(*args, **kwargs)



# Generated by Django 3.2.5 on 2021-07-25 11:04

from django.db import migrations
import tinymce.models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0023_assessment_pass_score'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assessment',
            name='description',
            field=tinymce.models.HTMLField(),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-24 16:31

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0014_candidateassessmentanswer_candidate_job'),
    ]

    operations = [
        migrations.CreateModel(
            name='CandidateAssessment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('unique_id', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('exam_started_at', models.DateTimeField(blank=True, null=True)),
                ('exam_end_at', models.DateTimeField(blank=True, null=True)),
                ('score', models.FloatField(default=0)),
                ('step', models.JSONField(blank=True, null=True)),
                ('exam_type', models.CharField(choices=[('mcq', 'MCQ Assessment'), ('written', 'Written Assessment'), ('viva', 'Viva Assessment')], max_length=50)),
                ('assessment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='job_board.assessment')),
                ('candidate_job', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='job_board.candidatejob')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
